/*******************************************************************************
 * Copyright (c) 2010- Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard;

import org.simantics.databoard.accessor.Accessor;
import org.simantics.databoard.accessor.ArrayAccessor;
import org.simantics.databoard.accessor.MapAccessor;
import org.simantics.databoard.accessor.RecordAccessor;
import org.simantics.databoard.accessor.error.AccessorConstructionException;
import org.simantics.databoard.accessor.error.AccessorException;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.binding.mutable.MutableVariantBinding;
import org.simantics.databoard.binding.mutable.Variant;
import org.simantics.databoard.datasource.IStepwiseDatasource;
import org.simantics.databoard.datasource.StepwiseDatasourceAccessor;

public class Datasources {

	/** Root node ID */
	public static final Variant ROOT_ID = new Variant();
	
	/**
	 * Open an accessor to a stepwise datasource.
	 * 
	 * @param datasource
	 * @return accessor
	 */
	public static StepwiseDatasourceAccessor getAccessor(IStepwiseDatasource datasource) {
		return new StepwiseDatasourceAccessor(datasource);
	}
	
	/**
	 * Get children of a node 
	 * 
	 * @param datasource
	 * @param nodeId
	 * @return ids of children
	 * @throws AccessorException
	 */
	public static Variant[] getChildren(Accessor datasource, Variant nodeId)
	throws AccessorException {
		Binding ID_BINDING = MutableVariantBinding.INSTANCE;
		try {
			RecordAccessor ra = (RecordAccessor) datasource;
			MapAccessor ma = ra.getFieldAccessor("nodes");
			RecordAccessor na = ma.getValueAccessor(ID_BINDING, nodeId);
			ArrayAccessor aa = na.getFieldAccessor("children");
			int size = aa.size();
			Variant[] result = new Variant[ size ];
			aa.getAll(ID_BINDING, result);
			return result;
		} catch (AccessorConstructionException e) {
			throw new AccessorException( e );
		}
	}
	
}

