/*******************************************************************************
 * Copyright (c) 2010- Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.accessor.emit;

import java.util.Collection;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.simantics.databoard.accessor.Accessor.Listener;
import org.simantics.databoard.accessor.event.Event;
import org.simantics.databoard.accessor.impl.EventCollection;

/**
 * Collecting emitter gathers events, one list for each listener, 
 * and emits on user's command.
 *
 * @author Toni Kalajainen <toni.kalajainen@vtt.fi>
 */
public class CollectingEmitter implements EventEmitter {

	Map<Listener, EventCollection> map = new IdentityHashMap<Listener, EventCollection>();
	
	/**
	 * Queue an event
	 * 
	 * @param listener
	 * @param event
	 */
	@Override
	public void emitEvent(Listener listener, Event event) {
		EventCollection cs = map.get(listener);
		if (cs==null) {
			cs = new EventCollection();
			map.put(listener, cs);
		}
		cs.addEvent(event);
	}

	/**
	 * Queue a collection of events
	 * 
	 * @param listener
	 * @param events
	 */
	@Override
	public void emitEvents(Listener listener, Collection<Event> events) {
		EventCollection cs = map.get(listener);
		if (cs==null) {
			cs = new EventCollection();
			map.put(listener, cs);
		}
		cs.addEvents(events);
	}
	
	/**
	 * Forward queued events to listeners.
	 */
	public void forwardEvents() {
		Iterator<Entry<Listener, EventCollection>> iter = map.entrySet().iterator();
		while (iter.hasNext()) {
			Entry<Listener, EventCollection> entry = iter.next();
			Listener l = entry.getKey();
			EventCollection cs = entry.getValue();
			if (cs.isEmpty()) {
				iter.remove();
			} else {
				l.onEvents( cs.getAndClearEvents() );
			}			
		}
	}

	/**
	 * Clear events
	 */
	public void clear() {
		map.clear();
	}

}

