/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.accessor.reference;

import org.simantics.databoard.util.URIUtil;


/**
 * A reference by name to the value of an union.
 *   
 * "t-<name>"
 *
 * @author Toni Kalajainen <toni.kalajainen@vtt.fi>
 */
public class TagNameReference extends AccessorReference {
	
	public String tag;
	
	public TagNameReference(String tagName) {
		this.tag = tagName;
	}
	
	public TagNameReference(String tagName, AccessorReference child) {
		super(child);
		this.tag = tagName;
	}
	
	@Override
	public String toString(boolean labelReference) {
		if (labelReference && (tag.equals("uv") || tag.equals("o") || tag.equals("v"))) labelReference = false;
		String encoded = URIUtil.encodeURI( tag );
		return labelReference ? encoded : "t-"+encoded; 
	}

	@Override
	public AccessorReference clone() {
		return new TagNameReference(tag, childReference==null ? null : childReference.clone());
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof TagNameReference == false) return false;
		TagNameReference other = (TagNameReference) obj;
		if (!other.tag.equals(tag)) return false;
		if (other.hasChildReference() != hasChildReference()) return false;
		if (hasChildReference() && !other.childReference.equals(childReference)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int hash = 1468996 + 37 * tag.hashCode();
		if (hasChildReference()) hash = 31*hash + childReference.hashCode();		
		return hash;
	}		
	
}

