/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.type;

import java.util.Set;

import org.simantics.databoard.util.IdentityPair;
import org.simantics.databoard.util.Limit;
import org.simantics.databoard.util.ObjectUtils;
import org.simantics.databoard.util.Range;

public class FloatType extends NumberType {
	
	public static final FloatType INSTANCE = new FloatType();
	    
    public FloatType() {}
    public FloatType(String unit) {
    	this.unit = unit;
    }
    public FloatType(String unit, Range range) {
    	this.unit = unit;
    	setRange(range);
    }    
    public FloatType(String unit, String range) {
    	this.unit = unit;
    	setRange(range);
    }    
    
    public float minValue() {
    	if (range==null) return -Float.MAX_VALUE;
    	Limit l = range.getLower();
    	float value = l.getValue().floatValue();
    	
    	// Won't work
    	if (l.isExclusive()) value+= Float.MIN_VALUE;
    	
    	return value;
    }
    
    public float maxValue() {
    	if (range==null) return Float.MAX_VALUE;
    	Limit l = range.getUpper();
    	float value = l.getValue().floatValue();
    	
    	// Won't work
    	if (l.isExclusive()) value-= Float.MIN_VALUE;
    	
    	return value;
    }    
       
    @Override
    protected boolean deepEquals(Object obj,
    		Set<IdentityPair<DataType, DataType>> compareHistory) {
		if (this==obj) return true;
		if (obj instanceof FloatType == false) return false;
		FloatType other = (FloatType) obj;
		return 
		ObjectUtils.objectEquals(other.range, range) &&
		ObjectUtils.objectEquals(other.unit, unit);
    }

    @Override
    public int hashCode() {
    	return 0x43676ee + 13*ObjectUtils.hashCode(range) + 7*ObjectUtils.hashCode(unit);
    }  
	@Override
	public void accept(Visitor1 v, Object obj) {
	    v.visit(this, obj);        
	}

	@Override
	public <T> T accept(Visitor<T> v) {
	    return v.visit(this);
	}	
	
}
