/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.util.binary;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

/**
 * IWriteable implementation with ByteBuffer as backend
 * 
 * @author Toni Kalajainen (toni.kalajainen@vtt.fi)
 */
public class ByteBufferWriteable implements BinaryWriteable, Seekable {

	ByteBuffer buf;
	
	public ByteBufferWriteable(ByteBuffer buf)
	{
		if (buf == null)
			throw new IllegalArgumentException("null");
		this.buf = buf;
	}

	@Override
	public void put(byte b) {
		buf.put(b);
	}

	@Override
	public void put(ByteBuffer src) {
		buf.put(src);
	}

	@Override
	public void put(ByteBuffer src, int length) {
		if (src.hasArray()) {
			byte[] array = src.array();
			buf.put(array, src.arrayOffset() + src.position(), length);
		} else {
			for (int i=0; i<length; i++)
				buf.put(src.get());
		}
	}

	@Override
	public void put(byte[] src, int offset, int length) {
		buf.put(src, offset, length);
	}

	@Override
	public void put(byte[] src) {
		buf.put(src);
	}

	@Override
	public void putDouble(double value) {
		buf.putDouble(value);
	}

	@Override
	public void putFloat(float value) {
		buf.putFloat(value);
	}

	@Override
	public void putInt(int value) {
		buf.putInt(value);
	}

	@Override
	public void putLong(long value) {
		buf.putLong(value);
	}

	@Override
	public void putShort(short value) {
		buf.putShort(value);
	}

	@Override
	public ByteOrder order() {
		return buf.order();
	}

	@Override
	public void order(ByteOrder order) {
		buf.order(order);
	}

	@Override
	public void flush() {
	}
	
	@Override
	public long position() throws IOException {
		return buf.position();
	}
	
	@Override
	public void position(long newPosition) throws IOException {
		if (newPosition>=Integer.MAX_VALUE || newPosition<0) throw new IllegalArgumentException("index out of range");
		buf.position((int) newPosition);		
	}
	
}

